package com.smppw.modaq.application.components.report.parser.ai;

import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.smppw.modaq.application.components.report.parser.AbstractReportParser;
import com.smppw.modaq.common.enums.ReportParseStatus;
import com.smppw.modaq.common.exception.ReportParseException;
import com.smppw.modaq.domain.dto.report.ReportData;
import com.smppw.modaq.domain.dto.report.ReportParserParams;
import com.smppw.modaq.domain.mapper.EmailFieldMappingMapper;
import org.springframework.beans.factory.annotation.Value;

import java.util.Map;

/**
 * 抽象的报告AI解析器
 *
 * @param <T> 泛型参数
 */
public abstract class AbstractAIReportParser<T extends ReportData> extends AbstractReportParser<T> {
    @Value("${email.report.ai-parser-url}")
    private String aiParserUrl;

    protected String aiFileId;

    protected Map<String, Object> infoMap;

    public AbstractAIReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    public T parse(ReportParserParams params) throws ReportParseException {
        // 初始化
        this.init();
        String filename = params.getFilename();
        String filepath = params.getFilepath();
        Map<String, Object> paramsMap = MapUtil.newHashMap(4);
        paramsMap.put("filepath", filepath);
        paramsMap.put("file_id", params.getAiFileId());
        String prompt = this.prompt();
        if (StrUtil.isNotBlank(prompt)) {
            paramsMap.put("user_msg", prompt);
        }
        String body = null;
        try {
            body = HttpUtil.get(this.aiParserUrl, paramsMap);
            this.handleAiResult(body);
        } catch (ReportParseException e) {
            this.logger.warn("{} ai解析失败，解析结果{}，错误原因：{}", filename, body, ExceptionUtil.stacktraceToString(e));
            throw e;
        } catch (Exception e) {
            this.logger.warn("报告{} 在AI解析时报错：{}", filename, ExceptionUtil.stacktraceToString(e));
            throw new ReportParseException(ReportParseStatus.AI_NOT_FOUND);
        }
        return this.buildReportData(params, filename);
    }


    @Override
    protected void cleaningReportData(T reportData) {
        // do something.
    }

    /**
     * AI解析的提示词
     *
     * @return 返回null就用默认的提示词
     */
    protected String prompt() {
        return null;
    }

    /**
     * 处理ai解析结果，方便构建结构化对象
     *
     * @param result ai解析结果
     */
    protected abstract void handleAiResult(String result) throws ReportParseException;

    @Override
    protected void init() {
        super.init();
        // 先初始化为null
        this.aiFileId = null;
        this.infoMap = MapUtil.newHashMap(128);
    }
}
